'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var rxjs = require('rxjs');
var React = require('react');

/*! *****************************************************************************
Copyright (c) Microsoft Corporation. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License"); you may not use
this file except in compliance with the License. You may obtain a copy of the
License at http://www.apache.org/licenses/LICENSE-2.0

THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
MERCHANTABLITY OR NON-INFRINGEMENT.

See the Apache Version 2.0 License for specific language governing permissions
and limitations under the License.
***************************************************************************** */
/* global Reflect, Promise */

var extendStatics = Object.setPrototypeOf ||
    ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
    function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };

function __extends(d, b) {
    extendStatics(d, b);
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
}

var subscribeToSink = function (sink, next, error) {
    return sink.subscribe({
        next: next,
        error: error
    });
};
var createObservable = function (subscribe) {
    return rxjs.Observable.create(function (Listener) {
        var unsubscribe = subscribe(Listener);
        return unsubscribe;
    });
};

var withEffects = function (handler, errorHandler) { return function (aperture) { return function (BaseComponent) {
    return /** @class */ (function (_super) {
        __extends(WithEffects, _super);
        function WithEffects(props, context) {
            var _this = _super.call(this, props, context) || this;
            _this.decoratedProps = {};
            _this.listeners = {
                mount: [],
                unmount: [],
                allProps: [],
                props: {},
                fnProps: {}
            };
            Object.keys(props).forEach(function (propName) {
                var prop = props[propName];
                if (typeof prop === 'function') {
                    _this.decorateProp(prop, propName);
                }
            });
            var mountObservable = createObservable(function (listener) {
                _this.listeners.mount = _this.listeners.mount.concat(listener);
                return function () { return _this.listeners.mount.filter(function (l) { return l !== listener; }); };
            });
            var unmountObservable = createObservable(function (listener) {
                _this.listeners.unmount = _this.listeners.unmount.concat(listener);
                return function () { return _this.listeners.unmount.filter(function (l) { return l !== listener; }); };
            });
            var createPropObservable = function (propName) {
                var listenerType = propName
                    ? typeof _this.props[propName] === 'function'
                        ? 'fnProps'
                        : 'props'
                    : 'allProps';
                return createObservable(function (listener) {
                    if (listenerType === 'allProps') {
                        listener.next(_this.props);
                        _this.listeners.allProps = _this.listeners.allProps.concat(listener);
                        return function () {
                            _this.listeners.allProps.filter(function (l) { return l !== listener; });
                        };
                    }
                    if (listenerType === 'props') {
                        listener.next(_this.props[propName]);
                    }
                    _this.listeners[listenerType][propName] = (_this.listeners[listenerType][propName] || []).concat(listener);
                    return function () {
                        _this.listeners[listenerType][propName].filter(function (l) { return l !== listener; });
                    };
                });
            };
            _this.component = {
                mount: mountObservable,
                unmount: unmountObservable,
                observe: function (propName) {
                    return createPropObservable(propName);
                }
            };
            var sinkObservable = aperture(_this.props)(_this.component);
            _this.sinkSubscription = subscribeToSink(sinkObservable, handler(_this.props), errorHandler ? errorHandler(_this.props) : undefined);
            return _this;
        }
        WithEffects.prototype.componentDidMount = function () {
            this.listeners.mount.forEach(function (l) { return l.next(undefined); });
        };
        WithEffects.prototype.componentWillReceiveProps = function (nextProps) {
            var _this = this;
            // Note: this will be replaced by getDerivedStateFromProps
            // but for now, we want to support React < 16.3.0
            Object.keys(nextProps).forEach(function (propName) {
                if (typeof _this.props[propName] === 'function' &&
                    nextProps[propName] !== _this.props[propName]) {
                    _this.decorateProp(nextProps[propName], propName);
                }
            });
        };
        WithEffects.prototype.componentDidUpdate = function (prevProps) {
            this.sendNext(prevProps);
        };
        WithEffects.prototype.componentWillUnmount = function () {
            this.listeners.unmount.forEach(function (l) { return l.next(undefined); });
            this.sinkSubscription.unsubscribe();
        };
        WithEffects.prototype.render = function () {
            return React.createElement(BaseComponent, Object.assign({}, this.props, this.decoratedProps));
        };
        WithEffects.prototype.sendNext = function (prevProps) {
            var _this = this;
            Object.keys(this.listeners.props).forEach(function (propName) {
                var prop = _this.props[propName];
                if (!prevProps || prevProps[propName] !== prop) {
                    _this.listeners.props[propName].forEach(function (l) { return l.next(prop); });
                }
            });
            this.listeners.allProps.forEach(function (l) { return l.next(_this.props); });
        };
        WithEffects.prototype.decorateProp = function (prop, propName) {
            var _this = this;
            this.decoratedProps[propName] = function () {
                var args = [];
                for (var _i = 0; _i < arguments.length; _i++) {
                    args[_i] = arguments[_i];
                }
                var listeners = _this.listeners.fnProps[propName] || [];
                listeners.forEach(function (l) { return l.next(args[0]); });
                return prop.apply(void 0, args);
            };
        };
        return WithEffects;
    }(React.PureComponent));
}; }; };

var identity = function (arg) { return arg; };
var compose = function () {
    var fns = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        fns[_i] = arguments[_i];
    }
    if (fns.length === 0) {
        return identity;
    }
    if (fns.length === 1) {
        return fns[0];
    }
    return function (arg) {
        return fns.reduceRight(function (previousResult, fn) { return fn(previousResult); }, arg);
    };
};

exports.withEffects = withEffects;
exports.compose = compose;
